@@ -17,17 +17,19 @@ template<class V, class Alloc>
 void write(c4::yml::NodeRef *n, std::vector<V, Alloc> const& vec)
 {
     *n |= c4::yml::SEQ;
-    for(auto const& v : vec)
+    for(V const& v : vec)
         n->append_child() << v;
 }
 
 template<class V, class Alloc>
 bool read(c4::yml::ConstNodeRef const& n, std::vector<V, Alloc> *vec)
 {
-    vec->resize(n.num_children());
+    C4_SUPPRESS_WARNING_GCC_WITH_PUSH("-Wuseless-cast")
+    vec->resize(static_cast<size_t>(n.num_children()));
+    C4_SUPPRESS_WARNING_GCC_POP
     size_t pos = 0;
-    for(auto const ch : n)
-        ch >> (*vec)[pos++];
+    for(ConstNodeRef const child : n)
+        child >> (*vec)[pos++];
     return true;
 }
 
@@ -36,12 +38,14 @@ bool read(c4::yml::ConstNodeRef const& n, std::vector<V, Alloc> *vec)
 template<class Alloc>
 bool read(c4::yml::ConstNodeRef const& n, std::vector<bool, Alloc> *vec)
 {
-    vec->resize(n.num_children());
+    C4_SUPPRESS_WARNING_GCC_WITH_PUSH("-Wuseless-cast")
+    vec->resize(static_cast<size_t>(n.num_children()));
+    C4_SUPPRESS_WARNING_GCC_POP
     size_t pos = 0;
-    bool tmp = false;
-    for(auto const ch : n)
+    bool tmp = {};
+    for(ConstNodeRef const child : n)
     {
-        ch >> tmp;
+        child >> tmp;
         (*vec)[pos++] = tmp;
     }
     return true;
